// File: controls.cpp
// Purpose: Generic
// 2015-09 jan@breem.nl

#include "TFTv2.h"
#include "LibTFT2p8.h"
#include "controls.h"

//******************************************************************************************************
void StoreTouch (int X, int Y, void (*CallBackPtr) (void))
{
  tft_StoreTouchArea (X + 10, Y + 10, X + 50, Y + 50, CallBackPtr);
}
 
//******************************************************************************************************
void ClearScreen (void)
{
  tft_ClearScreen();
}

//******************************************************************************************************
void DrawButtPlus (int X, int Y, int Color, void (*CallBackPtr) (void), bool EnterInTouchList)
{
  tft_DrawThickCircle (X+30, Y+30, 30, 3, Color); 
  tft_DrawThickVerticalLine (X+25, Y+15, 30, 10, Color);  
  tft_DrawThickHorizontalLine (X+15, Y+25, 30, 10, Color);
  if (EnterInTouchList) StoreTouch (X, Y, CallBackPtr); 
}

//******************************************************************************************************
void DrawButtMinus (int X, int Y, int Color, void (*CallBackPtr) (void), bool EnterInTouchList)
{
  tft_DrawThickCircle (X+30, Y+30, 30, 3, Color); 
  tft_DrawThickHorizontalLine (X+15, Y+25, 30, 10, Color);
  if (EnterInTouchList) StoreTouch (X, Y, CallBackPtr); 
}
 
 //******************************************************************************************************
void DrawButtOn (int X, int Y, int Color, void (*CallBackPtr) (void), bool EnterInTouchList)
{
  tft_DrawThickCircle (X+30, Y+30, 30, 3, Color); 
  tft_DrawThickVerticalLine (X+26, Y+15, 30, 10, Color);
  if (EnterInTouchList) StoreTouch (X, Y, CallBackPtr); 
}

//******************************************************************************************************
void DrawButtOff (int X, int Y, int Color, void (*CallBackPtr) (void), bool EnterInTouchList)
{
  tft_DrawThickCircle (X+30, Y+30, 30, 3, Color); 
  tft_DrawThickCircle (X+30, Y+30, 15, 5, Color); 
  if (EnterInTouchList) StoreTouch (X, Y, CallBackPtr);
}

//******************************************************************************************************
void DrawButtTimer (int X, int Y, int Color, void (*CallBackPtr) (void), bool EnterInTouchList)
{
  tft_DrawThickCircle (X+30, Y+30, 30, 3, Color); 
  tft_DrawThickVerticalLine(X+29, Y+10, 20, 3, Color);
  tft_DrawLine(X+29, Y+30, X+45, Y+46, Color);
  tft_DrawLine(X+30, Y+30, X+46, Y+46, Color);
  tft_DrawLine(X+31, Y+30, X+46, Y+45, Color);
  if (EnterInTouchList) StoreTouch (X, Y, CallBackPtr);
}

//******************************************************************************************************
void DrawButtPlay (int X, int Y, int Color, void (*CallBackPtr) (void), bool EnterInTouchList)
{
  tft_DrawThickRectangle (X, Y, 60, 60, 4, Color); 
  for (int n = 0; n < 30; n++)
    tft_DrawThickVerticalLine (X+45-n, Y+15+n/2, 30-n, 1, Color);
  if (EnterInTouchList) StoreTouch (X, Y, CallBackPtr);
}

//******************************************************************************************************
void DrawButtPause (int X, int Y, int Color, void (*CallBackPtr) (void), bool EnterInTouchList)
{
  tft_DrawThickRectangle (X, Y, 60, 60, 4, Color); 
  tft_DrawThickVerticalLine (X+15, Y+15, 30, 10, Color);
  tft_DrawThickVerticalLine (X+35, Y+15, 30, 10, Color); 
  if (EnterInTouchList) StoreTouch (X, Y, CallBackPtr);
}

//******************************************************************************************************
void DrawButtStop (int X, int Y, int Color, void (*CallBackPtr) (void), bool EnterInTouchList)
{
  tft_DrawThickRectangle (X, Y, 60, 60, 4, Color); 
  tft_DrawThickVerticalLine (X+15, Y+15, 30, 30, Color); 
  if (EnterInTouchList) StoreTouch (X, Y, CallBackPtr); 
}

//******************************************************************************************************
void DrawButtForward (int X, int Y, int Color, void (*CallBackPtr) (void), bool EnterInTouchList)
{
  tft_DrawThickRectangle (X, Y, 60, 60, 4, Color); 
  for (int n = 0; n < 30; n++)
  {
    tft_DrawThickVerticalLine (X+45-n, Y+15+n/3, 5, 1, Color);
    tft_DrawThickVerticalLine (X+45-n, Y+45-8-n/3, 5, 1, Color);
  }
  if (EnterInTouchList) StoreTouch (X, Y, CallBackPtr); 
}

//******************************************************************************************************
void DrawButtReverse (int X, int Y, int Color, void (*CallBackPtr) (void), bool EnterInTouchList)
{
  tft_DrawThickRectangle (X, Y, 60, 60, 4, Color); 
  for (int n = 0; n < 30; n++)
  {
    tft_DrawThickVerticalLine (X+15+n, Y+15+n/3, 5, 1, Color);
    tft_DrawThickVerticalLine (X+15+n, Y+45-8-n/3, 5, 1, Color);
  }
  if (EnterInTouchList) StoreTouch (X, Y, CallBackPtr);
}

//******************************************************************************************************
void DrawButtBack (int X, int Y, int Color, void (*CallBackPtr) (void), bool EnterInTouchList)
{
  tft_DrawThickCircle (X+30, Y+30, 30, 3, Color); 
  for (int n = 0; n < 30; n++)
  {
    tft_DrawThickVerticalLine (X+45-n, Y+15+n/3, 5, 1, Color);
    tft_DrawThickVerticalLine (X+45-n, Y+45-8-n/3, 5, 1, Color);
  }
  if (EnterInTouchList) StoreTouch (X, Y, CallBackPtr);
}

//******************************************************************************************************
void DrawButtZ (int X, int Y, int Color, void (*CallBackPtr) (void), bool EnterInTouchList)
{
  tft_DrawThickRectangle (X, Y, 60, 60, 4, Color); 
  tft_DrawChar ('Z', X+14, Y+6, 5, Color, BLACK);
  if (EnterInTouchList) StoreTouch (X, Y, CallBackPtr); 
}

//******************************************************************************************************
void DrawButtRD (int X, int Y, int Color, bool R, void (*CallBackPtr) (void), bool EnterInTouchList)
{
  tft_DrawThickRectangle (X, Y, 60, 60, 4, Color); 
  if (R) 
    tft_DrawChar ('R', X+14, Y+6, 5, CYAN, BLACK);
  else
    tft_DrawChar ('D', X+14, Y+6, 5, GREEN, BLACK); 
  if (EnterInTouchList) StoreTouch (X, Y, CallBackPtr); 
}

//******************************************************************************************************
void DrawButtRecordType (int X, int Y, t_RecordType RecordType, bool Enabled, void (*CallBackPtr) (void), bool EnterInTouchList)
{
  if (!Enabled)
  {
    tft_DrawThickRectangle (X, Y, 60, 60, 4, GRAY1); 
    switch (RecordType)
    {
    case LP: 
      tft_DrawThickCircle (X+30, Y+30, 25, 3, GREEN);
      tft_DrawThickCircle (X+30, Y+30, 19, 3, GRAY1);
      tft_DrawThickCircle (X+30, Y+30, 13, 3, GRAY1);
      break;
    case EP: 
      tft_DrawThickCircle (X+30, Y+30, 25, 3, GRAY1);
      tft_DrawThickCircle (X+30, Y+30, 19, 3, GREEN);
      tft_DrawThickCircle (X+30, Y+30, 13, 3, GRAY1);
      break;
    case SI: 
      tft_DrawThickCircle (X+30, Y+30, 25, 3, GRAY1);
      tft_DrawThickCircle (X+30, Y+30, 19, 3, GRAY1);
      tft_DrawThickCircle (X+30, Y+30, 13, 3, GREEN);
      break;  
    } 
  }  
  else
  {
    tft_DrawThickRectangle (X, Y, 60, 60, 4, GREEN); 
    switch (RecordType)
    {
    case LP: 
      tft_DrawThickCircle (X+30, Y+30, 25, 3, GREEN);
      tft_DrawThickCircle (X+30, Y+30, 19, 3, BLUE);
      tft_DrawThickCircle (X+30, Y+30, 13, 3, BLUE);
      break;
    case EP: 
      tft_DrawThickCircle (X+30, Y+30, 25, 3, BLUE);
      tft_DrawThickCircle (X+30, Y+30, 19, 3, GREEN);
      tft_DrawThickCircle (X+30, Y+30, 13, 3, BLUE);
      break;
    case SI: 
      tft_DrawThickCircle (X+30, Y+30, 25, 3, BLUE);
      tft_DrawThickCircle (X+30, Y+30, 19, 3, BLUE);
      tft_DrawThickCircle (X+30, Y+30, 13, 3, GREEN);
      break;  
    }  
  }  
  if (EnterInTouchList) StoreTouch (X, Y, CallBackPtr); 
}

//******************************************************************************************************
void DrawLed (int X, int Y, byte diam, bool on, int Color)
{
  if (on)
    tft_DrawFilledCircle (X, Y, 8, Color); 
  else
    tft_DrawThickCircle (X, Y, 8, 1, Color); 
}

//******************************************************************************************************
void UpdateVUmeter (byte Left, byte Right)
{
  static byte OldLeft, OldRight; 
  if (Left > OldLeft)
  {
    if (Left > 191)
    {
      tft_DrawThickHorizontalLine(250, OldLeft, 20, 192-OldLeft, GREEN);
      tft_DrawThickHorizontalLine(250, 192, 20, Left-191, RED);
    }
    else 
      tft_DrawThickHorizontalLine(250, OldLeft, 20, Left-OldLeft, GREEN);
  }  
  else
    tft_DrawThickHorizontalLine(250, Left, 20, OldLeft-Left+1, BLACK);
  OldLeft = Left;

  if (Right > OldRight)
  {
    if (Right > 191)
    {
      tft_DrawThickHorizontalLine(290, OldRight, 20, 192-OldRight, GREEN);
      tft_DrawThickHorizontalLine(290, 192, 20, Right-191, RED);
    }
    else 
      tft_DrawThickHorizontalLine(290, OldRight, 20, Right-OldRight, GREEN);
  }  
  else
    tft_DrawThickHorizontalLine(290, Left, 20, OldRight-Right+1, BLACK);
  OldRight = Right;  
}

//******************************************************************************************************
void InitVUmeter (void)
{
  tft_DrawThickVerticalLine(280, 0, 191, 1, WHITE);
  tft_DrawThickVerticalLine(280, 192, 239, 1, RED);
  for (byte i=0; i<8; i++)
    if (i<<5 > 191)
      tft_DrawThickHorizontalLine(273, i<<5, 15, 1, RED);
    else
      tft_DrawThickHorizontalLine(273, i<<5, 15, 1, WHITE);
}





